@@ -1,10 +1,8 @@
-from typing import Iterable, Tuple, Union
-from time import time
+from typing import Iterable, Tuple
 
 import matplotlib.pyplot as plt
 import statsmodels.api as sm
-from pydts.base_fitters import ExpansionBasedFitter
-from sklearn.model_selection import train_test_split
+from .base_fitters import ExpansionBasedFitter
 from scipy.optimize import minimize
 from scipy.special import logit, expit
 import numpy as np
@@ -13,18 +11,18 @@ from lifelines.fitters.coxph_fitter import CoxPHFitter
 from pandarallel import pandarallel
 from typing import Optional, List, Union
 from matplotlib import colors as mcolors
-from tqdm import tqdm
 from joblib import Parallel, delayed
-from pydts.examples_utils.generate_simulations_data import generate_quick_start_df
 
+from .examples_utils.generate_simulations_data import generate_quick_start_df
+from .utils import assert_fit
 
 COLORS = list(mcolors.TABLEAU_COLORS.keys())
 
 
 class DataExpansionFitter(ExpansionBasedFitter):
     """
-    This class implements the fitter as described in Lee et al. 2018 [1]
-    See also Simple Example section.
+    This class implements the estimation procedure of Lee et al. (2018) [1].
+    See also the Example section.
 
     Example:
         ```py linenums="1"
@@ -35,7 +33,7 @@ class DataExpansionFitter(ExpansionBasedFitter):
         ```
 
     References:
-        [1] "On the Analysis of Discrete Time Competing Risks Data", Lee et al., Biometrics, 2018, DOI: 10.1111/biom.12881
+        [1] Lee, Minjung and Feuer, Eric J. and Fine, Jason P., "On the analysis of discrete time competing risks data", Biometrics (2018) doi: 10.1111/biom.12881
     """
 
     def __init__(self):
@@ -168,8 +166,7 @@ class DataExpansionFitter(ExpansionBasedFitter):
 class TwoStagesFitter(ExpansionBasedFitter):
 
     """
-    This class implements the new approach for fitting model to discrete time survival data.
-    See also Simple Example section.
+    This class implements the approach of Meir et al. (2022):
 
     Example:
         ```py linenums="1"
@@ -178,6 +175,9 @@ class TwoStagesFitter(ExpansionBasedFitter):
             fitter.fit(df=train_df, event_type_col='J', duration_col='X')
             fitter.print_summary()
         ```
+
+    References:
+    [1] Meir, Tomer\*, Gutman, Rom\*, and Gorfine, Malka, "PyDTS: A Python Package for Discrete-Time Survival Analysis with Competing Risks" (2022)
     """
 
     def __init__(self):
@@ -225,13 +225,13 @@ class TwoStagesFitter(ExpansionBasedFitter):
 
         Args:
             df (pd.DataFrame): training data for fitting the model
-            covariates (list): list of covariates to be used in fitting the beta model
+            covariates (list): list of covariates to be used in estimating the regression coefficients
             event_type_col (str): The event type column name (must be a column in df),
-                                  Right censored sample (i) is indicated by event value 0, df.loc[i, event_type_col] = 0.
+                                  Right-censored sample (i) is indicated by event value 0, df.loc[i, event_type_col] = 0.
             duration_col (str): Last follow up time column name (must be a column in df).
             pid_col (str): Sample ID column name (must be a column in df).
             x0 (Union[numpy.array, int], Optional): initial guess to pass to scipy.optimize.minimize function
-            fit_beta_kwargs (dict, Optional): Keyword arguments to pass on to fit beta procedure.
+            fit_beta_kwargs (dict, Optional): Keyword arguments to pass on to the estimation procedure.
                                               If different model for beta is desired, it can be defined here.
                                               For example:
                                               fit_beta_kwargs={
@@ -281,7 +281,7 @@ class TwoStagesFitter(ExpansionBasedFitter):
                                     row[duration_col])), axis=1)
             n_et['success'] = n_et['opt_res'].parallel_apply(lambda val: val.success)
             n_et['alpha_jt'] = n_et['opt_res'].parallel_apply(lambda val: val.x[0])
-            assert_fit(n_et, self.times)  # todo move basic input validation before any optimization
+            assert_fit(n_et, self.times, event_type_col=event_type_col, duration_col=duration_col)  # todo move basic input validation before any optimization
             self.event_models[event] = [self.beta_models[event], n_et]
             self.alpha_df = pd.concat([self.alpha_df, n_et], ignore_index=True)
         return self.event_models
@@ -299,14 +299,17 @@ class TwoStagesFitter(ExpansionBasedFitter):
         Returns:
             None
         """
+        from IPython.display import display
+        display(self.get_beta_SE())
+
         for event, model in self.event_models.items():
-            _summary_func = getattr(model[0], summary_func, None)
-            if _summary_func is not None:
-                print(f'\n\nModel summary for event: {event}')
-                _summary_func(**summary_kwargs)
-            else:
-                print(f'Not {summary_func} function in event {event} model')
-            from IPython.display import display
+            print(f'\n\nModel summary for event: {event}')
+            # _summary_func = getattr(model[0], summary_func, None)
+            # if _summary_func is not None:
+            #     print(f'\n\nModel summary for event: {event}')
+            #     _summary_func(**summary_kwargs)
+            # else:
+            #     print(f'Not {summary_func} function in event {event} model')
             display(model[1].drop('opt_res', axis=1).set_index([self.event_type_col, self.duration_col]))
 
     def plot_event_alpha(self, event: Union[str, int], ax: plt.Axes = None, scatter_kwargs: dict = {},
@@ -502,95 +505,74 @@ class TwoStagesFitter(ExpansionBasedFitter):
         return ax
 
 
-def create_df_for_cif_plots(df: pd.DataFrame, field: str,
-                            covariates: Iterable,
-                            vals: Optional[Iterable] = None,
-                            quantiles: Optional[Iterable] = None,
-                            zero_others: Optional[bool] = False
-                            ) -> pd.DataFrame:
-    """
-    This method creates df for cif plot, where it zeros
-
-    Args:
-        df (pd.DataFrame): Dataframe which we yield the statiscal propetrics (means, quantiles, etc) and stacture
-        field (str): The field which will represent the change
-        covariates (Iterable): The covariates of the given model
-        vals (Optional[Iterable]): The values to use for the field
-        quantiles (Optional[Iterable]): The quantiles to use as values for the field
-        zero_others (bool): Whether to zero the other covarites or to zero them
-
-    Returns:
-        df (pd.DataFrame): A dataframe that contains records per value for cif ploting
-    """
-
-    cov_not_fitted = [cov for cov in covariates if cov not in df.columns]
-    assert len(cov_not_fitted) == 0, \
-        f"Required covariates are missing from df: {cov_not_fitted}"
-    # todo add assertions
-
-    df_for_ploting = df.copy()  # todo make sure .copy() is required
-    if vals is not None:
-        pass
-    elif quantiles is not None:
-        vals = df_for_ploting[field].quantile(quantiles).values
-    else:
-        raise NotImplemented("Only Quantiles or specific values is supported")
-    temp_series = []
-    template_s = df_for_ploting.iloc[0][covariates].copy()
-    if zero_others:
-        impute_val = 0
-    else:
-        impute_val = df_for_ploting[covariates].mean().values
-    for val in vals:
-        temp_s = template_s.copy()
-        temp_s[covariates] = impute_val
-        temp_s[field] = val
-        temp_series.append(temp_s)
-
-    return pd.concat(temp_series, axis=1).T
-
-
-def assert_fit(event_df, times):
-    # todo: split to 2: one generic, one for new model
-    if not event_df['success'].all():
-        problematic_times = event_df.loc[~event_df['success'], "X"].tolist()
-        event = event_df['J'].max()  # all the events in the dataframe are the same
-        raise RuntimeError(f"In event J={event}, The method did not converged in D={problematic_times}."
-                           f" Consider changing the problem definition."
-                           f"\n See https://tomer1812.github.io/pydts/User%20Story/ for more details.")
-    if event_df.shape[0] != len(times):
-        event = event_df['J'].max()  # all the events in the dataframe are the same
-        problematic_times = pd.Index(event_df['X']).symmetric_difference(times).tolist()
-        raise RuntimeError(f"In event J={event}, The method didn't have events D={problematic_times}."
-                           f" Consider changing the problem definition."
-                           f"\n See https://tomer1812.github.io/pydts/User%20Story/ for more details.")
-
-
-def repetitive_fitters(rep, n_patients, n_cov, d_times, j_events, pid_col, test_size,
+def repetitive_fitters(rep: int, n_patients: int, n_cov: int, d_times: int, j_events: int, pid_col: str,
+                       test_size: float,
                        drop_cols: Iterable = ("C", "T"),
-                       model1=DataExpansionFitter,
+                       model1: ExpansionBasedFitter = DataExpansionFitter,
                        model1_name="Lee",
-                       model2=TwoStagesFitter,
+                       model2: ExpansionBasedFitter = TwoStagesFitter,
                        model2_name: str = "Ours",
                        allow_fails: int = 20,
-                       verbose: int = 2
-                       ) -> Tuple[dict, dict]:
-    # todo docstrings
-    # todo assertions
-    # todo move to utils?
-    # todo try catch
+                       verbose: int = 2,
+                       real_coef_dict: dict = None,
+                       censoring_prob: float = 1.
+                       ) -> Tuple[dict, dict, pd.DataFrame]:
+    """
+    The function allows the user to generate N repetitions of model training (given data generating process),
+    to allow the user to compare the parameters stability and fitting time of the methods .
 
-    from pydts.examples_utils.plots import compare_beta_models_for_example
+    Args:
+        rep (int): number of repetitions to run the models
+        n_patients (int): number of sample to generate for each repetition
+        n_cov (int): number of covariates to generate for each repetition
+        d_times (int): how many times T to generate (i.e. times would be $t \in [1,\ldots,d]$)
+        j_events (int): number of events to generate for each repetition
+        pid_col (str): the name of the id column
+        test_size (float): the test size (percentage) for train/test splitting
+        drop_cols (Iterable):
+            Columns that shouldn't be visible for the model, from the generated sample.
+            default is drop the real event time (T) and censoring time (C)
+        model1 (ExpansionBasedFitter): Typically Lee et al. 2018 [1] model (DataExpansionFitter).
+            but can be any of base type ExpansionBasedFitter
+        model1_name (str): the name of the first model
+        model2 (ExpansionBasedFitter): Typically our suggested method (TwoStagesFitter).
+            but can be any of base type ExpansionBasedFitter
+        model2_name  (str): the name of the second model
+        allow_fails (int): number of allowed failed repetitions.
+            I.e. the method would run up to rep + allow_fails times.
+        verbose (int): verbosity level for the pandarallel module
+        real_coef_dict (dict): dictionary which represent the real coefficients to be generated for each repetition
+        censoring_prob (float): The probability to use the censoring method for each round of generation
+
+    Returns:
+        rep_dict (dict): Dictionary which contains for each round (key)
+            its beta/alpha/real coefficients dataframe (value).
+        times (dict): Dictionary which  contains for each model (key) a list of the training times (value).
+        ret_df (pd.DataFrame): Dataframe which contains for each time in d times, the average amount of events for it.
+
+    """
+    # todo assertions
+
+    from .examples_utils.plots import compare_beta_models_for_example
+    from sklearn.model_selection import train_test_split
+    from tqdm import trange
+    from time import time
+    assert real_coef_dict is not None, "The user should supply the coefficients of the experiment"
     rep_dict = {}
     times = {model1_name: [], model2_name: []}
     counts_df_list = []
     final = 0
-    for samp in tqdm(range(rep+allow_fails)):
+    failed = 0
+    for samp in trange(rep+allow_fails):
         try:
-            patients_df = generate_quick_start_df(n_patients=n_patients, n_cov=n_cov, d_times=d_times, j_events=j_events,
-                                                  pid_col=pid_col, seed=samp)
+            patients_df = generate_quick_start_df(n_patients=n_patients, n_cov=n_cov, d_times=d_times,
+                                                  j_events=j_events,
+                                                  pid_col=pid_col, seed=samp, real_coef_dict=real_coef_dict,
+                                                  censoring_prob=censoring_prob )
             train_df, test_df = train_test_split(patients_df, test_size=test_size)
-            counts_df_list.append(train_df.groupby(['J', 'X']).size().to_frame(samp))
+            counts_df = train_df.groupby(['J', 'X']).size().to_frame(samp)
+            assert not (counts_df.reset_index()['X'].value_counts() < (j_events + 1)).any(), "Not enough events"
+            counts_df_list.append(counts_df)
             drop_cols = pd.Index(drop_cols)
             start_1 = time()
             fitter = model1()
@@ -605,66 +587,17 @@ def repetitive_fitters(rep, n_patients, n_cov, d_times, j_events, pid_col, test_
             end_2 = time()
             times[model1_name].append(end_1 - start_1)
             times[model2_name].append(end_2 - start_2)
-            res_dict = compare_beta_models_for_example(fitter.event_models, new_fitter.event_models)
+            res_dict = compare_beta_models_for_example(fitter.event_models,
+                                                       new_fitter.event_models, real_coef_dict=real_coef_dict)
             rep_dict[samp] = res_dict
             final += 1
             if final == rep:
                 break
-        except:
-            print(f'Failed to fit sample {samp}')
+        except Exception as e:
+            print(e)
+            failed += 1
+            print(f'Failed to fit sample {samp+1}, fail #{failed}')
             continue
     print(f'final: {final}')
     ret_df = pd.concat(counts_df_list, axis=1).fillna(0).mean(axis=1).apply(np.ceil).to_frame()
-    return rep_dict, times, ret_df
-
-
-def get_real_hazard(df, real_coef_dict, times, events):
-    """
-
-    Args:
-        df:
-        real_coef_dict:
-        times:
-        events:
-
-    Returns:
-
-    """
-    # todo docstrings
-    # todo assertions
-    # todo move to utils?
-
-    a_t = {event: {t: real_coef_dict['alpha'][event](t) for t in times} for event in events}
-    b = pd.concat([df.dot(real_coef_dict['beta'][j]) for j in events], axis=1, keys=events)
-
-    for j in events:
-        df[[f'hazard_j{j}_t{t}' for t in times]] = pd.concat([expit(a_t[j][t] + b[j]) for t in times],
-                                                             axis=1).values
-    return df
-
-
-if __name__ == "__main__":
-    from pydts.examples_utils.generate_simulations_data import generate_quick_start_df
-    n_patients = 50000
-    n_cov = 5
-    patients_df = generate_quick_start_df(n_patients=n_patients, n_cov=n_cov, d_times=30, j_events=2,
-                                          pid_col='pid', seed=0)
-    train_df, test_df = train_test_split(patients_df, test_size=0.25)
-    # m = DataExpansionFitter()
-    # m.fit(df=df.drop(['C', 'T'], axis=1))
-    # m.print_summary()
-
-    m2 = TwoStagesFitter()
-    m2.fit(train_df.drop(['C', 'T'], axis=1))
-    #m2.plot_all_events_alpha()
-    #pred_df = m2.predict_hazard_all(test_df)
-    #pred_df = m2.predict_overall_survival(test_df, t=5)
-    #pred_prob = m2.predict_prob_event_j_at_t(test_df, event=1, t=2)
-    #m2.predict_event_cumulative_incident_function(test_df, event=1)
-    tdf = test_df[[f'Z{i+1}' for i in range(n_cov)]]
-    m2.predict_cumulative_incident_function(tdf)
-    # m2.predict(test_df)
-    # print(m2.get_beta_SE())
-    # m2.plot_all_events_beta()
-    print('x')
-
+    return rep_dict, times, ret_df
\ No newline at end of file
